bl_info = {
    "name": "MaSzyna Developer Tools",
    "decription": "Import and Export MaSzyna models (T3D, E3D) and sceneries (SCN, SCM)",
    "author": "Krzysztof Przybyla (krzysiuup), Kamil Lewan (carmel4a), Grzegorz Firlejczyk (firleju)",
    "version": (1, 0, 0),
    "blender": (2, 78, 0),
    "location": "File > Import-Export",
    #"warning": "This add-on is still under developement. Be aware of some errors that might occur.",
    "category": "Import-Export",
}

import collections

import bpy
from bpy.props import (
    StringProperty,
    BoolProperty,
    )
from bpy_extras.io_utils import (
    ExportHelper,
    ImportHelper,
    )

T3DExportConfig = collections.namedtuple("ExportConfig",
    ["filepath",
    "use_selection",
    "use_image_ext",
    "use_mesh_modifiers",
    "apply_transform",
    "fix_geometry"]
    )

class ImportT3D(bpy.types.Operator, ImportHelper):
    """Import MaSzyna T3D & E3D model files"""
    bl_idname = "import_model.t3d"
    bl_label = 'Import T3D/E3D'
    filename_ext = ".t3d"
    filter_glob = StringProperty(
            default="*.t3d;*.e3d",
            options={'HIDDEN'},
            )

    def execute(self, context):
        try:
            from maszyna_dev_tools import import_t3d
        except ImportError:
            print("T3D Import module is not avaliable.")
        else:
            import_t3d.import_(self.filepath)
        return {'FINISHED'}

class ExportT3D(bpy.types.Operator, ExportHelper):
    """Export MaSzyna T3D model file"""
    bl_idname = "export_model.t3d"
    bl_label = 'Export T3D'
    bl_options = {'PRESET'}
    filename_ext = ".t3d"
    filter_glob = StringProperty(
            default="*.t3d",
            options={'HIDDEN'},
            )
    use_selection = BoolProperty(
            name="Selection Only",
            description="Export selected objects only",
            default=False,
            )
    use_image_ext = BoolProperty(
            name="Image extension",
            description="Write with image extensions",
            default=False,
            )
    use_mesh_modifiers = BoolProperty(
            name="Apply Modifiers",
            description="Apply modifiers (preview resolution)",
            default=True,
            )
    apply_transform = BoolProperty(
            name="Apply transform",
            description="Provide identity transform matrix to all objects",
            default=True,
            )
    fix_geometry = BoolProperty(
            name="Fix geometry",
            description="Fix geometry by removing doubles and dissolving degenerate faces.",
            default=True,
            )

    def execute(self, context):
        try:
            from maszyna_dev_tools import export_t3d
        except ImportError:
            print("T3D Export module is not avaliable.")
        else:
            self.filepath = bpy.path.ensure_ext(self.filepath, self.filename_ext)
            config = T3DExportConfig(
                self.filepath,
                self.use_selection,
                self.use_image_ext,
                self.use_mesh_modifiers,
                self.apply_transform,
                self.fix_geometry,
                )
            export_t3d.export(config)
        return {'FINISHED'}

class ImportSCN(bpy.types.Operator, ImportHelper):
    """Imports MaSzyna SCN scenery file"""
    bl_idname = "import_scenery.scn"
    bl_label = 'Import SCN'
    filename_ext = ".scn"
    filter_glob = StringProperty(
            default="*.scn;*.scm",
            options={'HIDDEN'},
            )
    def execute(self, context):
        try:
            from maszyna_dev_tools import import_scn
        except ImportError:
            print("SCN Import module is not avaliable.")
        else:
            import_scn.import_(self.filepath)
        return {'FINISHED'}

class ExportSCN(bpy.types.Operator, ExportHelper):
    """Exports MaSzyna SCN scenery file"""
    bl_idname = "export_scenery.scn"
    bl_label = 'Export SCN'
    bl_options = {'PRESET'}
    filename_ext = ".scn"
    filter_glob = StringProperty(
            default="*.scn;*.scm",
            options={'HIDDEN'},
            )

    _option_items = [
    ("Export mesh", "Export mesh", "", 1)
    ]
    def set_items(self , value):
        pass
    option_ = bpy.props.EnumProperty(
        items = _option_items,
        name="Mode",
        set=set_items
        )
    def execute(self, context):
        export_scn(self.filepath, self.option_)
        return {'FINISHED'}

class ImportASCII(bpy.types.Operator, ImportHelper):
    """ASCII(xyz) vertex cloud"""
    bl_idname = "import_ascii.txt"
    bl_label = 'Import T3D/E3D'
    filename_ext = ".txt"
    filter_glob = StringProperty(
            default="*.txt",
            options={'HIDDEN'},
            )

    def execute(self, context):
        try:
            from maszyna_dev_tools import import_ascii_xyz
        except ImportError:
            print("NMT Import module is not avaliable.")
        else:
            import_ascii_xyz.import_(self.filepath)
        return {'FINISHED'}

class AddonProperties(bpy.types.AddonPreferences):
    bl_idname = __name__
    root_path = bpy.props.StringProperty(
        name ="MaSzyna catalog",
        description = "A root catalog with MaSzyna simulator",
        default = "",
        maxlen=0,
        options={'ANIMATABLE'},
        subtype='DIR_PATH'
        )

    def draw(self, context):
        self.layout.prop(self,"root_path")

class AddonPropertiesOperator(bpy.types.Operator):
    bl_idname = "object.addon_prefs"
    bl_label = "Add-on Preferences"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        addon_prefs = context.user_preferences.addons["maszyna_dev_tools"].preferences
        self.report({'INFO'}, (addon_prefs.root_path))
        return {"FINISHED"}

def menu_func_import(self, context):
    self.layout.operator(ImportT3D.bl_idname, text="MaSzyna (.t3d, .e3d)")
    self.layout.operator(ImportSCN.bl_idname, text="MaSzyna (.scn, .scm)")
    self.layout.operator(ImportASCII.bl_idname, text="NMT (.txt)")

def menu_func_export(self, context):
    self.layout.operator(ExportT3D.bl_idname, text="MaSzyna (.t3d)")
    self.layout.operator(ExportSCN.bl_idname, text="MaSzyna (.scn, .scm)")

def register():
    bpy.utils.register_module(__name__)
    bpy.types.INFO_MT_file_import.append(menu_func_import)
    bpy.types.INFO_MT_file_export.append(menu_func_export)

def unregister():
    bpy.utils.unregister_module(__name__)
    bpy.types.INFO_MT_file_import.remove(menu_func_import)
    bpy.types.INFO_MT_file_export.remove(menu_func_export)
if __name__ == '__main__':
    register()
