import abc
import math
import os

import bpy

class _ExportStrategy(abc.ABC):
    def __init__(self):
        self.t3d_code = ""
        self.submodel_type = ""

    def get_t3d_code(self, obj: bpy.types.Object):
        self.save_separator()
        self.save_first_common_properties(obj)
        self.save_specific_properties(obj)
        self.save_last_common_properties(obj)
        self.save_verts(obj)
        return self.t3d_code

    def save_separator(self):
        NUMBER_OF_DASHES = 81
        self.t3d_code += "//{}\n".format("".join(["-"]*NUMBER_OF_DASHES))

    def save_first_common_properties(self, obj):
        self._save_property(
            "Parent",
            obj.parent.name if obj.parent is not None else "none"
            )
        self._save_property(
            "Type",
            self.submodel_type
            )
        self._save_property(
            "Name",
            obj.name
            )
        self._save_property(
            "Anim",
            obj.get("Anim", "false")
            )

    def save_last_common_properties(self, obj):
        print("MAKSDISANCE: {}".format(obj["MaxDistance"]) if "MaxDistance" in obj.keys() else "")
        self._save_property(
            "MaxDistance",
            obj.get("MaxDistance", "-1")
            )

        self._save_property(
            "MinDistance",
            obj.get("MinDistance", "0")
            )

        self._save_property(
            "Transform",
            "\n" + "\n".join(["{:19.6f} {:.6f} {:.6f} {:.6f}".format(*row) for row in (obj.matrix_basis * obj.matrix_parent_inverse.transposed())])
            )

    def _save_property(self, name, value):
        self.t3d_code += "{}: {}\n".format(name, value)

    @abc.abstractmethod
    def save_specific_properties(self, obj):
        pass

    @abc.abstractmethod
    def save_verts(self, obj):
        pass

class MeshExportStrategy(_ExportStrategy):
    def __init__(self):
        super().__init__()
        self.submodel_type = "Mesh"

    def save_specific_properties(self, obj):
        has_material = bool(obj.active_material)
        self.has_texture = bool(obj.active_material.active_texture) if has_material else False

        self._save_property(
            "Ambient",
            " ".join([str(float(math.floor(obj.active_material.ambient * 255)))] * 3) if has_material else "0.0 0.0 0.0"
            )
        self._save_property(
            "Diffuse",
            " ".join([str(float(math.floor(obj.active_material.diffuse_color[0] * 255)))] * 3) if has_material else "0.0 0.0 0.0"
            )
        self._save_property(
            "Specular",
            " ".join([str(float(math.floor(obj.active_material.specular_color[0] * 255)))] * 3) if has_material else "0.0 0.0 0.0"
            )
        self._save_property(
            "SelfIllum",
            "false" if not has_material else "true" if obj.active_material.emit == 1 else "false" if obj.active_material.emit == 0 else obj.active_material.emit * 100
            )
        self._save_property(
            "Wire",
            "true" if obj.draw_type == 'WIRE' else "false"
            )
        self._save_property(
            "WireSize",
            "1.0"
            )
        self._save_property(
            "Opacity",
            obj.active_material.alpha if has_material and obj.active_material.use_transparency else 100.0
            )
        self._save_property(
            "Map",
            self._get_texture_path(obj) if self.has_texture else "none"
            )

    def save_verts(self, obj):
        bpy.context.scene.objects.active = obj
        bpy.ops.object.modifier_add(type="TRIANGULATE")
        triangulated_mesh = obj.to_mesh(bpy.context.scene, self.config.use_mesh_modifiers, "PREVIEW")

        self._save_property(
            "NumVerts",
            len(triangulated_mesh.polygons)*3
            )

        for triangle in triangulated_mesh.polygons:
            self.t3d_code += "{}\n".format(
                triangulated_mesh.calc_smooth_groups(True)[0][triangle.index] if triangle.use_smooth else 0
                )

            for i, vert_idx in enumerate(triangle.vertices):
                uv_coords = triangulated_mesh.uv_layers.active.data[triangle.loop_indices[i]].uv
                self.t3d_code +="{:.6f} {:.6f} {:.6f} {:.6f} {:.6f}\n".format(
                    *triangulated_mesh.vertices[vert_idx].co,
                    *uv_coords if self.has_texture else (0,0)
                    )
            self.t3d_code += "\n"

        obj.modifiers.remove(obj.modifiers[-1])

    def _get_texture_path(self, obj):
        if not "replacableskin" in obj.keys():
            full_path = obj.active_material.active_texture.image.filepath_from_user()
            relative_path = os.path.relpath(full_path, full_path[:full_path.index("{}textures".format(os.sep))+9])
            relative_path = os.sep + relative_path if relative_path.count(os.sep) == 0 else relative_path
            relative_path = relative_path if self.config.use_image_ext else os.path.splitext(relative_path)[0]
        else:
            print("BENDZIE REPL")
            relative_path = obj["replacableskin"] if obj["replacableskin"] in ["-1", "-2", "-3", "-4"] else "replacableskin"

        return relative_path

class FreespotlightExportStrategy(_ExportStrategy):
    def __init__(self):
        super().__init__()
        self.submodel_type = "FreeSpotLight"

    def save_specific_properties(self, obj):
        self._save_property(
            "Diffuse",
            " ".join(map(lambda x: str(x * 255), obj.color))
            )
        self._save_property(
            "SelfIllum",
            "true"
            )
        self._save_property(
            "NearAttenStart",
            "0.0"
            )
        self._save_property(
            "NearAttenEnd",
            "40.0"
            )
        self._save_property(
            "UseNearAtten",
            "false"
            )
        self._save_property(
            "FarAttenDecayType",
            "1" if obj.data.falloff_type == 'INVERSE_LINEAR' else "2" if obj.data.falloff_type == 'INVERSE_SQUARE' else "0"
            )
        self._save_property(
            "FarDecayRadius",
            obj.data.distance
            )
        self._save_property(
            "FalloffAngle",
            math.degrees(obj.data.spot_size * (1+obj.data.spot_blend))
            )
        self._save_property(
            "HotspotAngle",
            math.degrees(obj.data.spot_size)
            )

    def save_verts(self, obj):
        pass

class StarsExportStrategy(_ExportStrategy):
    pass
